import student.takequiz as tq
import teacher.checkstudentscores as css
# from student import takequiz as tq
# from teacher import checkstudentscores as css

def start():
    print("Are you a student or a professor?")
    print("1. Student \n2. Teacher")
    role = input("Enter response:")

    match role:
        case "1":
            tq.select_quiz()
        case "2":
            css.quiz_or_score()
        case _ :
            print("Please enter valid input")     



import string
import pandas as pd
import json

# Constants
QUIZ_FILE_DIRECTORY = "../assets/quizzes/"
UNDERSCORE = '_'
SCORE_FILE_DIRECTORY = "../assets/scores/"
SCORES = 'scores'
q = ""
o = []
a = ""

class Question():
    """ This class bundles the quiz inputs into a dictionary"""
    all_questions = []

    def __init__(self, question, options, answer):
        self.quiz = {}
        self.question = question
        self.options = options
        self.answer = answer
    
    def write_to_dict(self):
        """ Create a dictionary from the question class attributes """
        self.quiz["question"] = self.question
        self.quiz["options"] = self.options
        self.quiz["answer"] = self.answer
        Question.all_questions.append(self.quiz)
        return Question.all_questions

class questionInput():
    """
    Provides user inputed quiz question, options, and answers to the Question class.
    
    Takes the user input for questions, options and the answer and creates an object for the same.
    
    Returns
    ----------
    list
        list of question, options and answer
    """
    def quiz_inputs(self):
        self.question = input("Input Question: ")
        self.options = []
        self.answer = ""
        self.answer_added = False
        num_options = 1
        for letter in string.ascii_uppercase:
            option = input(f"{letter}. Enter option: ")
            self.options.append(f"{letter}. {option}")
            if self.answer_added == False:
                print("Is this the correct answer? ('y' for yes, any other key for no)")
                response = input("RESPONSE: ")
                if response.lower() == "y":
                    self.answer = f"{letter}. {option}"
                    self.answer_added = True
            response = print("Would you like to input another option? ('y' for yes, any other key for no)")
            response = input("RESPONSE: ")
            if response.lower() != "y":
                break
            else:
                num_options += 1

        return [self.question, self.options, self.answer]
    
def convert_to_json(course, questions):
    """
    Converts quiz dictionary to json object and writes object into json file.
    
    Parameters
    ----------
    string : course
        course code
    list : questions
        quiz questions
    """ 
    filename = QUIZ_FILE_DIRECTORY + course + ".json"
    with open(filename, "w") as out_file:
        json.dump(questions, out_file, indent=6)  

def create_score_csv(course):
    """
    Create empty csv file to store all student quiz grades for a course.
    
    Parameters
    ----------
    string : course
        course code
    """ 
    filename = SCORE_FILE_DIRECTORY + course + UNDERSCORE + SCORES + ".csv"
    data = ["student_number", "student_name", "score"]
    df = pd.DataFrame(columns = data)
    df.to_csv(filename, index = False)

def create_quiz():
    """
    Driver function to create a quiz and write quiz into a file as a json object.
    
    This serves as adriver function to create a quiz and then
    writes the quiz into a JSON file in the appropriate directory
    with the appropriate filename.
    """ 

    print("Do you want to create a quiz?('y' for yes, any other key for no)")
    response = input("RESPONSE: ")
    if response.lower() == "y":
        while True:
            q_contents = questionInput().quiz_inputs()
            q = Question(q_contents[0], q_contents[1], q_contents[2])
            questions = q.write_to_dict()
            print("Do you want to add another question?")
            print("'y' for yes, any other key for no")
            response = input("response: ")
            if response.lower() != "y":
                break
        course = input("Input the course code: ")
        convert_to_json(course, questions)
        create_score_csv(course)
    else:
        print("Not creating quiz")





from . import readquiz as rq
import random

class Student:
    """ This class bundles student name and student number into individual quiz objects """
    def __init__(self, student_name, student_number) -> None:
        self.student_name = student_name
        self.student_number = student_number
        
def student_handler():
    """ Returns a student object with user inputs of student name and student number """
    student_name = input("Please enter your name:")
    student_number = input("Please enter your student number:")
    return Student(student_name, student_number)

def start_quiz(quiz):
    """
    Student start quiz
    
    It is the driver function that enables a student to enter a quiz, answer questions, and obtain a quiz score.
    
    Parameters
    ----------
    object : quiz
        course code
    
    Returns
    -------
    float : score
    """
    score = 0
    problems = quiz.problems
    for i in range(len(problems)):
        no_of_questions = len(problems)
        ch = random.randint(0, no_of_questions-1)
        print(f'\nQ{i+1}. {problems[ch].question}\n')
        for option in problems[ch].options:
            print(option)
        answer = input("\nEnter your answer: ")
        if problems[ch].answer[0] == answer[0].upper():
            print("\nYou are correct")
            score+=1
        else:
            print("\nYou are incorrect")
        del problems[ch]
    score = round(score / quiz.no_of_problems * 100, 2)
    return score

def select_quiz():
    """
    Select a quiz 
    
    It displays all available quizzes then prompts a student to select a quiz to complete. Then it calculates and
    displays overall quiz average via get_percentage() in the readquiz module if the student prompts the function.
    """
    student = student_handler()
    print("Welcome", student.student_name)
    print("Quizzes for these courses are available:")
    courses = rq.get_available_courses()
    course_dict = {}
    for i in range(len(courses)):
        print(i+1, '.', courses[i])
        course_dict[i+1] = courses[i]
    course_input = int(input("Enter your choice:"))
    if course_input not in list(course_dict.keys()):
        print("No such quiz available.")
        return
    course_code = course_dict.get(course_input)
    quiz = rq.read_quiz(course_code)
    score = start_quiz(quiz)
    success = rq.save_score(student, course_code, score)
    if success:
        print("Your score is:", score, "and it was saved successfully.")
    print("Do you want to calculate overall percentage?")
    print("1. Yes \n2. No")
    calc_percentage_flag = int(input("Enter response:"))
    if calc_percentage_flag == 1:
        percentage = rq.get_percentage(student)
        print("Your overall percentage across all courses is:", percentage)


import os
import pandas as pd
import numpy as np
import statistics as stats
from teacher import createquiz as cq

# Constants
SCORE_FILE_DIRECTORY = "../assets/scores/"
UNDERSCORE = '_'
SCORES = 'scores'


def get_student_scores(course, stdnum):
    """
    View a students quiz results
    
    It displays the student's quiz score for a course by taking student number
    and course code as arguments.
    
    Parameters
    ----------
    string : course
        course code
    string : stdnum
        student number of the student
    """
    filename = SCORE_FILE_DIRECTORY + course + UNDERSCORE + SCORES + ".csv"
    if os.path.exists(filename):
        scores_table = pd.read_csv(filename, skipinitialspace = True)
        stdnum_list = list(scores_table["student_number"])
        if stdnum in stdnum_list:
            location = np.where(scores_table["student_number"] == stdnum)[0][0]
            name = scores_table.at[location, "student_name"]
            stdnum = str(scores_table.at[location, "student_number"])
            score = str(scores_table.at[location, "score"])
            print("\n" + "\n" + "Name: " + name + "\n" +
                "Student Number: " + str(stdnum) + "\n" +
                "Score: " + score + "\n" + "\n")
        else:
            print("\n" + "\n" + str(stdnum) + " is not found in the score file" + "\n" + "\n")
    else:
        print("\n" + "\n" + "Sorry, no quizzes have been attempted yet for " + course + "\n" + "\n")


def set_student_scores(course, student_number, score):
    """
    Modify student score if an error in quiz is present
    
    It modifies the student's score if there is an error in the quiz.
    
    Parameters
    ----------
    string : course
        course code
    string : student_number
        student number of the student
    string : score
        score of the student which needs to be updated
    """
    filename = SCORE_FILE_DIRECTORY + course + UNDERSCORE + SCORES + ".csv"
    if os.path.exists(filename):
        scores_table = pd.read_csv(filename, skipinitialspace = True)
        stdnum_list = list(scores_table["student_number"])
        if student_number in stdnum_list:
            if 0 <= score <= 100:
                scores_table.loc[scores_table.student_number == student_number, "score"] = score
                location = np.where(scores_table["student_number"] == student_number)[0][0]
                name = scores_table.at[location, "student_name"]
                print("\n" + "\n" + "updated mark for " + name + ": " + str(score) + "\n" + "\n")
                scores_table.to_csv(filename, index = False)
            else:
                print("Updated mark must be between 0 and 100 percent")
        else:
            print("\n" + "\n" + str(student_number) + " is not found in the score file" "\n" + "\n")
    else:
        print("\n" + "\n" + "Sorry, no quizzes have been attempted yet for " + course + "\n" + "\n")


def quiz_score_statistics(course):
    """
    Calculate and display average course grade of all students
    
    It calculates and display metrics like average, minimum, maximum etc.
    for the course grade of all students.
    
    Parameters
    ----------
    string : course
        course code
    """
    filename = SCORE_FILE_DIRECTORY + course + UNDERSCORE + SCORES + ".csv"
    if os.path.exists(filename):
        scores_table = pd.read_csv(filename, skipinitialspace = True)
        scores = scores_table["score"]
        minimum = str(min(scores))
        lowerQuartile = str(scores.quantile([0.25])).split()[1]
        average = str(stats.mean(scores))
        median = str(stats.median(scores))
        upperQuartile = str(scores.quantile([0.75])).split()[1]
        maximum = str(max(scores))
        print("\n" + "\n" +"MINIMUM: " + minimum + "\n" +
            "LOWER QUARTILE: " + lowerQuartile + "\n" +
            "MEAN: " + average + "\n" +
            "MEDIAN: " + median + "\n" +
            "UPPER QUARTILE: " + upperQuartile + "\n" +
            "MAXIMUM: " + maximum + "\n" + "\n")
    else:
        print("Sorry, no quizzes have been appempted yet for " + course)


def score_driver():
    """
    Driver code directing teacher to view a students mark,
    modify a students mark, calculate quiz summary statistics, or leave the program
    """
    while True:
        print("What would you like to do?")
        print("1. View students quiz mark \n2. Change a students quiz mark \n3. Calculate quiz score summary statistics \n4. Quit")
        response = input("response: ")
        match response:
            case "1":
                course = input("Provide the course code: ")
                stnum = int(input("Provide student number: "))
                get_student_scores(course, stnum)
            case "2": 
                course = input("Provide the course code: ")
                stnum = int(input("Provide student number: ")) 
                mark = int(input("Provide updated mark: "))
                set_student_scores(course, stnum, mark)
            case "3":
                course = input("Provide the course code: ")
                quiz_score_statistics(course)
            case "4":
                break
            case _:
                print("invalid option")


def quiz_or_score():
    """ 
    Driver code directing teacher to create a quiz,
    check quiz score marks, or leave the program
    """
    while True:
        print("Do you want to create a quiz or check quiz score marks?")
        print("1. Create Quiz \n2. See marks \n3. Quit")
        response = input("response: ")
        match response:
            case "1":
                cq.create_quiz()
            case "2":
                score_driver()
            case "3":
                break
            case _:
                print("Please enter valid input")


import unittest

from readquiz_test import ReadQuizTest
from takequiz_test import TakeQuizTest
from createquiz_test import CreateQuizTest
# from checkstudentscore_test import CheckStudentScoreTest
# from main_test import MainTest

def quizapp_test_suite():
    suite = unittest.TestSuite()
    result = unittest.TestResult()
    suite.addTest(ReadQuizTest('test_read_quiz'))
    suite.addTest(ReadQuizTest('test_get_available_courses'))
    suite.addTest(ReadQuizTest('test_save_score'))
    suite.addTest(ReadQuizTest('test_get_percentage'))
    suite.addTest(TakeQuizTest('test_student_handler'))
    suite.addTest(TakeQuizTest('test_select_quiz'))
    suite.addTest(CreateQuizTest("test_create_quiz"))
    suite.addTest(CreateQuizTest("test_convert_to_json"))
    suite.addTest(CreateQuizTest("test_create_score_csv"))
    suite.addTest(CreateQuizTest("test_create_quiz"))
    # Tests work, issue with python version in Travis for these:  
    
    # suite.addTest(CheckStudentScoreTest("test_score_driver"))
    # suite.addTest(CheckStudentScoreTest("test_quiz_or_score"))
    # suite.addTest(MainTest("test_start"))
    # suite.addTest(MainTest("test_start_2"))
    # suite.addTest(MainTest("test_start_3"))
    runner = unittest.TextTestRunner()
    print(runner.run(suite))
quizapp_test_suite()


import json
import random
import os
import csv

from os.path import isfile, join

# Constants
QUIZ_FILE_DIRECTORY = "../assets/quizzes/"
SCORE_FILE_DIRECTORY = "../assets/scores/"
UNDERSCORE = '_'
SCORES = 'scores'

# Set current directory to read from absolute path
abspath = os.path.abspath(__file__)
dname = os.path.dirname(abspath)
os.chdir(dname)

class Quiz:
    """ This class bundles question, problem, and total number of problems into individual quiz objects"""
    def __init__(self, name, problems):
        self.name = name
        self.problems = problems
        self.no_of_problems = len(problems)
        
class Problem:
    """ This class bundles individual questions, multiple choice options, and answers into individual problem objects"""
    def __init__(self, ques, options, ans):
        self.question = ques
        self.options = options
        self.answer = ans

# Read quiz from json
def read_quiz(course_code):
    """
    Read a quiz
    
    It outputs a quiz to the screen by taking course code as an argument.
    
    Parameters
    ----------
    string : course_code
        course code
    """

    file_name = QUIZ_FILE_DIRECTORY + course_code + ".json"
    problems = []
    with open(file_name, 'r+') as f:
        j = json.load(f)
        for i in range(len(j)):
            no_of_questions = len(j)
            ch = random.randint(0, no_of_questions-1)
            problem = Problem(j[ch]["question"], j[ch]["options"], j[ch]["answer"][0])
            problems.append(problem)
            del j[ch]
    quiz = Quiz(course_code, problems)
    return quiz

def get_available_courses():
    """
    Get a list of available courses
    
    It displays the all courses a student has access to in order to start.
    
    Returns
    ----------
    list : course_names
        a list of all courses
    """
    filedir = QUIZ_FILE_DIRECTORY
    files = [f for f in os.listdir(filedir) if f.endswith('.json')]
    course_names = [f.split('.', 1)[0] for f in files]
    return course_names

def save_score(student, course_code, score):
    """
    Save quiz score
    
    It saves the quiz score for a student into a csv file by taking student, course code, and score as arguments.
    
    Parameters
    ----------
    object : student
        course code
    string : course_code
        course code
    int : score
        quiz score

    Returns
    ----------
    boolean : True
    """
    score_row = [student.student_number, student.student_name, score]
    filename = SCORE_FILE_DIRECTORY + course_code + UNDERSCORE + SCORES + ".csv"
    with open(filename, 'a') as csvfile:
        scorewriter = csv.writer(csvfile)
        scorewriter.writerow(score_row)
        csvfile.close()
    return True

def get_percentage(student):
    """
    Get quiz score percentage
    
    It displays the student's quiz score averaged across quizzes as a percentage by taking student as an argument.
    
    Parameters
    ----------
    object : student
        student name
    
    Returns
    --------
    float : average_percentage
        average score across quizzes
    """
    courses = get_available_courses()
    total_scored = 0.0
    number_of_courses = 0
    average_percentage = 0.0
    for course in courses:
        filename = SCORE_FILE_DIRECTORY + course + UNDERSCORE + SCORES + ".csv"
        with open(filename) as score_file:
            reader = csv.DictReader(score_file)
            for row in reader:
                if row['student_number'] == student.student_number:
                    total_scored += float(row['score'])
                    number_of_courses += 1
    if number_of_courses != 0:
        average_percentage = total_scored / number_of_courses
    return average_percentage
                
        